// CRain.cpp

#include "CRain.h"
#include "CParticleEmitter.h"
#include <gl.h>

#include <math.h>

#define PIE 3.1415926
#define DEGREE2RAD PIE/180



//----------------------------------------------------------------------------------------
// CRain()
//----------------------------------------------------------------------------------------
CRain::CRain(CVector* Location)
{
	this->angle							= 25;
	this->numActiveParticles			= 0;
	this->maxParticles					= 650;
	this->EmitLocation					= Location;
	this->ParticleList					= new CParticle[this->maxParticles];
//	memset(ParticleList, 0, sizeof(CParticleEmitter)*maxParticles);		// Clear all the data
	
	for(int i=0; i<maxParticles; i++)
	{
		this->Initialize(numActiveParticles++);
	}
}


//----------------------------------------------------------------------------------------
// ~CRain()
//----------------------------------------------------------------------------------------
CRain::~CRain()
{
	delete [] ParticleList;
}


//----------------------------------------------------------------------------------------
// Initialize()
//----------------------------------------------------------------------------------------
int CRain::Initialize(unsigned int index)
{
	ParticleList[index].spread			= 100.0f;
	ParticleList[index].Position.x		= rand()%1024;
	ParticleList[index].Position.y		= (float)(500.0f + rand()%200);
	ParticleList[index].Position.z		= rand()%1024;
	ParticleList[index].size			= (float)(100.0f + rand()%45);
	ParticleList[index].velocity		= -20.5f;
	ParticleList[index].color			= 1.0f;
	ParticleList[index].is_active		= true;

 return 0;
}


//----------------------------------------------------------------------------------------
// Update()
//----------------------------------------------------------------------------------------
int CRain::Update(void)
{
	// Random chance that wind will change
	if(rand()%500 >= 490)
		this->angle = rand()%60;

	// If (requisite time has elapsed)
	// Create new particles
	for(unsigned int i=0; i <this->maxParticles; i++)
	{
		if(ParticleList[i].is_active == true)
		{
			ParticleList[i].Position.y -= (float)cos(this->angle * DEGREE2RAD) - ParticleList[i].velocity;
			ParticleList[i].Position.x -=  (float)cos(this->angle * DEGREE2RAD); //- ParticleList[i].velocity;

			if((ParticleList[i].Position.y - (1-(float)sin(this->angle * DEGREE2RAD)) * ParticleList[i].size) 
				<= myTerrain->getHeight(ParticleList[i].Position.x, ParticleList[i].Position.z))
			{
				ParticleList[i].Position.x		= rand()%1024;
				ParticleList[i].Position.y		= (float)(500.0f + rand()%200);
				ParticleList[i].Position.z		= rand()%1024;
			}
		}
	}
 
 return 0;
}

//----------------------------------------------------------------------------------------
// Render()
//----------------------------------------------------------------------------------------
int CRain::Render(void)
{

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);

	glEnable(GL_LINE_SMOOTH);
	glLineWidth(0.2f);
	glColor3f(0.0f, 0.0f, 0.8f);
	for(unsigned int i=0; i <this->maxParticles; i++)
	{
		glBegin(GL_LINES);
		if(this->ParticleList[i].is_active)
		{
			glVertex3f(ParticleList[i].Position.x,
					   ParticleList[i].Position.y,
					   ParticleList[i].Position.z);

			glVertex3f(ParticleList[i].Position.x - (1-(float)cos(this->angle * DEGREE2RAD)) * ParticleList[i].size,
					   ParticleList[i].Position.y - (1-(float)sin(this->angle * DEGREE2RAD)) * ParticleList[i].size,
					   ParticleList[i].Position.z);
		}
		glEnd();
	}

	glDisable(GL_BLEND);

 return 0;
}
