/******************************************************************************************************************
//
//
//
//
//
//
/******************************************************************************************************************/


#include "CTexture.h"
#include <fstream.h>
#include <glu.h>
#include <stdlib.h>
#include <stdio.h>


//----------------------------------------------------------------------------------------
// CTexture()
//----------------------------------------------------------------------------------------
CTexture::CTexture()
{
	this->width			= 0;
	this->height		= 0;
}


//----------------------------------------------------------------------------------------
// ~CTexture()
//----------------------------------------------------------------------------------------
CTexture::~CTexture()
{
//	delete Bitmap;
}


//----------------------------------------------------------------------------------------
// Load()
// Desc: Loads the desired bitmap, sets up mipmaps, then makes 'ID' a reference to the 
// texture data
//----------------------------------------------------------------------------------------
int CTexture::Load(char* filename)
{
	this->buffer = LoadBitmap(filename);

	glGenTextures(1, &this->ID);
	glBindTexture(GL_TEXTURE_2D, this->ID);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, this->width, this->height, 
				 0, GL_RGBA, GL_UNSIGNED_BYTE, this->buffer);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, this->width, this->height, 
					  GL_RGBA, GL_UNSIGNED_BYTE, this->buffer);

 return 0;
}


//----------------------------------------------------------------------------------------
// LoadBitmap()
//
// Loads a 24-bit bitmap from a file.
// Note: this function performs almost NO error checking, and makes
// a lot of assumptions of the correctness of the bitmap file
//----------------------------------------------------------------------------------------
unsigned char* CTexture::LoadBitmap(char *filename)
{
	ifstream BitmapFile;
	BitmapFile.open(filename, ios::binary| ios::nocreate, NULL);

	if(!BitmapFile.is_open())
	{
	//	char* error_string = "Could not open bitmap file: ";
	//	strncat(error_string, filename, sizeof(filename));
		MessageBox(NULL, "Could not open .bmp file", "You suck", MB_OK);
	}

	CBitmap Bitmap;

	Bitmap.bitmapfileheader = new BITMAPFILEHEADER;		// allocate memory for fileheader
	BitmapFile.read((char*)Bitmap.bitmapfileheader, sizeof(BITMAPFILEHEADER));		// read data into fileheader
	
	Bitmap.bitmapinfoheader = new BITMAPINFOHEADER;		// allocate memory for infoheader
	BitmapFile.read((char*)Bitmap.bitmapinfoheader, sizeof(BITMAPINFOHEADER));		// read data into infoheader

	width = Bitmap.bitmapinfoheader->biWidth;
	height = Bitmap.bitmapinfoheader->biHeight;

	int bytes_per_line = Bitmap.bitmapinfoheader->biWidth*3;

	Bitmap.data = new unsigned char[Bitmap.bitmapinfoheader->biSizeImage];	// allocate memory for buffer
	BitmapFile.read((char *)Bitmap.data, Bitmap.bitmapinfoheader->biSizeImage);	// read data into buffer

	unsigned char tempRGB = 0;
	// swap the R and B values to get RGB since the bitmap color format is in BGR
	for (int i=0; i < Bitmap.bitmapinfoheader->biSizeImage; i+=3)
	{
		tempRGB = Bitmap.data[i];
		Bitmap.data[i] = Bitmap.data[i + 2];
		Bitmap.data[i + 2] = tempRGB;
	}

	BitmapFile.close();

	unsigned char* alphaBitmap = new unsigned char[Bitmap.bitmapinfoheader->biSizeImage*4/3];
    for (int src = 0, dst = 0; src < Bitmap.bitmapinfoheader->biSizeImage; src +=3, dst +=4)
    {
		// if the pixel is black, set the alpha to 0. Otherwise, set it to 255.
		if (Bitmap.data[src] == 0 && Bitmap.data[src+1] == 0 && Bitmap.data[src+2] == 0)
			alphaBitmap[dst+3] = 0;
		else
			alphaBitmap[dst+3] = 0xFF;

		// copy pixel data over
		alphaBitmap[dst]	= Bitmap.data[src];
		alphaBitmap[dst+1]  = Bitmap.data[src+1];
		alphaBitmap[dst+2]  = Bitmap.data[src+2];
    }

	delete Bitmap.bitmapfileheader;
	delete Bitmap.bitmapinfoheader;
	delete Bitmap.data;

 return alphaBitmap;
}


