// CWinHandler.cpp

// INCLUDES
#include <windows.h>
#include <winuser.h>
#include <string.h>
#include <stdlib.h>
#include <gl.h>
#include <time.h>
#include "CWinHandler.h"


//----------------------------------------------------------------------------------------
// CWinHandler()
//----------------------------------------------------------------------------------------
CWinHandler::CWinHandler()
{
	ClassName = "RPGClass";
	bExit = false;
	Hwnd					= NULL;
	Hrc						= NULL;
	Hdc						= NULL;
	rScreenArea.left		= NULL;
	rScreenArea.top			= NULL;
	rScreenArea.right		= NULL;
	rScreenArea.bottom		= NULL;
}


//----------------------------------------------------------------------------------------
// ~CWinHandler()
//----------------------------------------------------------------------------------------
CWinHandler::~CWinHandler()
{
}


//----------------------------------------------------------------------------------------
// MessageHandler()
//
// Takes care of all windows-specific messages
//----------------------------------------------------------------------------------------
void CWinHandler::MessageHandler(void)
{
	if(PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
		{
			if(Msg.message == WM_QUIT)
				bExit = true;
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}
}


//----------------------------------------------------------------------------------------
// MakeWindowClass()
// Desc: Creates the windows class, then registers it
//----------------------------------------------------------------------------------------
int CWinHandler::MakeWindowClass(WNDPROC WinProc, HINSTANCE *hInstance)
{
	pHinstance = hInstance;
	
	wcMain.style = CS_VREDRAW | CS_HREDRAW | CS_OWNDC | CS_DBLCLKS;
	wcMain.lpfnWndProc = WinProc;
	wcMain.cbClsExtra = 0;
	wcMain.cbWndExtra = 0;
	wcMain.hInstance = *hInstance;
	wcMain.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wcMain.hCursor = LoadCursor((HINSTANCE) NULL, IDC_ARROW);
	wcMain.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH); 
	wcMain.lpszMenuName =  NULL;
	wcMain.lpszClassName = this->ClassName;

	// register it
	if(!RegisterClass(&wcMain))
	{
		//myError.Write_Error("Could not register windows class.");
		return 0;
	}

 return 1;
}




//----------------------------------------------------------------------------------------
// SetupScreen()
// Desc: Sets up appropriate resolution, color depth, and makes window full screen
//----------------------------------------------------------------------------------------
int CWinHandler::SetupScreen(int width, int height, int color_depth, bool b_full_screen)
{

	// if we're in fullscreen mode, set the display up for it
	if (b_full_screen)
	{
		// set up the device mode structure
		DEVMODE Screen;
		memset(&Screen,0,sizeof(Screen));

		Screen.dmSize       = sizeof(Screen);	
		Screen.dmPelsWidth  = width;			
		Screen.dmPelsHeight = height;			
		Screen.dmBitsPerPel = color_depth;		
		Screen.dmFields     = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

		ChangeDisplaySettings(&Screen, CDS_FULLSCREEN);
	} 

 return 1;
}


//----------------------------------------------------------------------------------------
// MakeWindow()
// Desc: Creates the main window
//----------------------------------------------------------------------------------------
int CWinHandler::MakeWindow(HINSTANCE *m_instance)
{
	ShowCursor(FALSE);
	
	rScreenArea.top    =  0; 
	rScreenArea.left   =  0;
	rScreenArea.bottom =  1024;
	rScreenArea.right  =  1280;

	Hwnd = CreateWindowEx(WS_EX_APPWINDOW, this->ClassName, "4E3 Entry", 
						  WS_POPUP,
						  rScreenArea.left, 
						  rScreenArea.top,
						  rScreenArea.right - rScreenArea.left,
						  rScreenArea.bottom - rScreenArea.top, 
						  NULL, NULL, *m_instance, NULL);
 return 1;
}


//----------------------------------------------------------------------------------------
// SetupPixelFormat()
// Desc: 
//----------------------------------------------------------------------------------------
int CWinHandler::SetupPixelFormat(void)
{

	// Get handle to device context
//	Hdc = GetDC(Hwnd);
   
	// set the pixel format we want
	PIXELFORMATDESCRIPTOR PixelFormat = 
	{
		sizeof(PIXELFORMATDESCRIPTOR),	// size of structure
		1,                              // default version
		PFD_DRAW_TO_WINDOW |            // window drawing support
		PFD_SUPPORT_OPENGL |            // OpenGL support
		PFD_DOUBLEBUFFER,               // double buffering support
		PFD_TYPE_RGBA,                  // RGBA color mode
		32,                           // 32 bit color mode
		0, 0, 0, 0, 0, 0,               // ignore color bits, non-palettized mode
		0,                              // no alpha buffer
		0,                              // ignore shift bit
		0,                              // no accumulation buffer
		0, 0, 0, 0,                     // ignore accumulation bits
		16,                             // 16 bit z-buffer size
		0,                              // no stencil buffer
		0,                              // no auxiliary buffer
		PFD_MAIN_PLANE,                 // main drawing plane
		0,                              // reserved
		0, 0, 0 
	};                      // layer masks ignored
      
	GLuint  format;

	// choose best matching pixel format
	format = ChoosePixelFormat(Hdc, &PixelFormat);

	// set pixel format to device context
	SetPixelFormat(Hdc, format,&PixelFormat);

	// create the OpenGL rendering context
	Hrc = wglCreateContext(Hdc);

	// now make the rendering context the active one
	wglMakeCurrent(Hdc, Hrc);

	// show the window in the forground, and set the keyboard focus to it
	ShowWindow(Hwnd, SW_SHOW);
	SetForegroundWindow(Hwnd);
	SetFocus(Hwnd);

 return 1;
}



