//**********************************************************************************************
// Project:  4E3 Entry
// Author: Sean Estey
// Date: 07.15.02
// Comments:
// All code is property of Sean Estey (and portions borrowed from OpenGL Game Programming), but 
// is available to the public domain with the permission of the author (me!).
// All I ask is that you give me credit if my engine (or game) is modified and/or distributed.
// Unauthorized use and/or distribution is strictly prohibited, and is punishable by death. 
// That is all.
// 
// WinMain.cpp
//**********************************************************************************************


// DEFINES
#define WIN32_LEAN_AND_MEAN 


// INCLUDES
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <windows.h>
#include <windowsx.h>
#include <gl.h>
#include "CEngine.h"
#include "CWorld.h"
#include "CWinHandler.h"
#include "CParticleEmitter.h"
#include <glaux.h>


// GLOBALS
CEngine* myEngine = new CEngine();
CWorld* myWorld = new CWorld();

// FUNCTION PROTOTYPES
LRESULT APIENTRY WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);


//----------------------------------------------------------------------------------------
// WinMain()
//
// Creates a windows class, registers it, creates window, runs through main event loop
//----------------------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hinstance, 
				   HINSTANCE hPrevInstance, 
				   LPSTR lpCmdLine, 
				   int nShowCmd)
{
	myEngine->myWin->MakeWindowClass(WindowProc, &hinstance);
  	myEngine->myWin->MakeWindow(&hinstance);
	myEngine->Initialize();
	myWorld->Init();




	while(!myEngine->myWin->getExit())
	{
		myEngine->myTimer->Init();
		glClearColor(0.1f, 0.3f, 0.5f, 1.0f);
		myEngine->myWin->MessageHandler();
		myWorld->Animate();
		myWorld->Render();
		SwapBuffers(myEngine->myWin->getHdc());
		glFlush();
		myEngine->myTimer->LockFPS(30);
	}	

 return(myEngine->myWin->getMsg().wParam);
}



//----------------------------------------------------------------------------------------
// WindowProc()
//
// 
//----------------------------------------------------------------------------------------
LRESULT APIENTRY WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	PAINTSTRUCT ps;
//	HDC hdc;

	switch(msg)
	{
		case WM_ACTIVATEAPP:
		{
			if (wparam)
			{	
				myEngine->myWin->SetupScreen(1280, 1024, 32, true);
			}
		}
		break;
		
		case WM_CREATE:
		{
			myEngine->myWin->setHdc(GetDC(hwnd));
			myEngine->myWin->setHwnd(hwnd);
			myEngine->myWin->SetupPixelFormat();
			// do initialization stuff here
			return 0;
		}
		break;

		case WM_PAINT:
		{
			BeginPaint(hwnd, &ps);
			EndPaint(hwnd, &ps);

			return 0;
		}
		break;

		case WM_DESTROY:
		{
			PostQuitMessage(0);

			return 0;
		}
		break;


		case WM_SIZE:
		{
			int height = HIWORD(lparam);		// retrieve width and height
			int width = LOWORD(lparam);

			if (height==0)					// don't want a divide by zero
				height=1;					
			
			glViewport(0, 0, width, height);	// reset the viewport to new dimensions
			glMatrixMode(GL_PROJECTION);		// set projection matrix current matrix
			glLoadIdentity();					// reset projection matrix

			// calculate aspect ratio of window
			gluPerspective(54.0f,(GLfloat)width/(GLfloat)height,1.0f,10000.0f);

			glMatrixMode(GL_MODELVIEW);			// set modelview matrix
			glLoadIdentity();					// reset modelview matrix

			return 0;
			break;
		}

		case WM_MOUSEMOVE:
		{

/*			// save old mouse coordinates
			oldMouseX = mouseX;
			oldMouseY = mouseY;

			// get mouse coordinates from Windows
			int mouseX = LOWORD(lparam);
			int mouseY = HIWORD(lparam);

			// these lines limit the camera's range
//			if (mouseY < 200)
//				mouseY = 200;
//			if (mouseY > 450)
//				mouseY = 450;

			if ((mouseX - oldMouseX) > 0)		// mouse moved to the right
				angle += 3.0f;
			else if ((mouseX - oldMouseX) < 0)	// mouse moved to the left
				angle -= 3.0f;

			return 0;
			break;*/
		}break;
		

		case WM_KEYDOWN:
		{
			switch(wparam)
			{
				case VK_ESCAPE:
				{
					PostQuitMessage(0);
				
					return 0;   
				}

				case VK_LEFT:
				{
					myWorld->myCamera->MoveTo(new CVector(myWorld->myCamera->getPosition()->x - 10,
														  myWorld->myCamera->getPosition()->y,
														  myWorld->myCamera->getPosition()->z));
					myWorld->myCamera->LookAt(new CVector(myWorld->myCamera->getLook()->x - 10,
														  myWorld->myCamera->getLook()->y,
														  myWorld->myCamera->getLook()->z));
					return 0;
				}

				case VK_UP:
				{
					myWorld->myCamera->MoveTo(new CVector(myWorld->myCamera->getPosition()->x,
														  myWorld->myCamera->getPosition()->y,
														  myWorld->myCamera->getPosition()->z - 10));
					myWorld->myCamera->LookAt(new CVector(myWorld->myCamera->getLook()->x,
														  myWorld->myCamera->getLook()->y,
														  myWorld->myCamera->getLook()->z - 10));
					return 0;
				}

				case VK_DOWN:
				{
					myWorld->myCamera->MoveTo(new CVector(myWorld->myCamera->getPosition()->x,
														  myWorld->myCamera->getPosition()->y,
														  myWorld->myCamera->getPosition()->z + 10));
					myWorld->myCamera->LookAt(new CVector(myWorld->myCamera->getLook()->x,
														  myWorld->myCamera->getLook()->y,
														  myWorld->myCamera->getLook()->z + 10));
					return 0;
				}

				case VK_RIGHT:
				{
					myWorld->myCamera->MoveTo(new CVector(myWorld->myCamera->getPosition()->x + 10,
														  myWorld->myCamera->getPosition()->y,
														  myWorld->myCamera->getPosition()->z));
					myWorld->myCamera->LookAt(new CVector(myWorld->myCamera->getLook()->x + 10,
														  myWorld->myCamera->getLook()->y,
														  myWorld->myCamera->getLook()->z));
					return 0;
				}

				case VK_NUMPAD1:
				{
					myWorld->myCamera->MoveTo(new CVector(myWorld->myCamera->getPosition()->x,
														  myWorld->myCamera->getPosition()->y + 10,
														  myWorld->myCamera->getPosition()->z));
					myWorld->myCamera->LookAt(new CVector(myWorld->myCamera->getLook()->x,
														  myWorld->myCamera->getLook()->y + 10,
														  myWorld->myCamera->getLook()->z));
					return 0;
				}


				case VK_NUMPAD2:
				{
					myWorld->myCamera->MoveTo(new CVector(myWorld->myCamera->getPosition()->x,
														  myWorld->myCamera->getPosition()->y - 10,
														  myWorld->myCamera->getPosition()->z));
					myWorld->myCamera->LookAt(new CVector(myWorld->myCamera->getLook()->x,
														  myWorld->myCamera->getLook()->y - 10,
														  myWorld->myCamera->getLook()->z));
					return 0;
				}



			}
		}
		break;

		default: break;
	}

 return (DefWindowProc(hwnd, msg, wparam, lparam));

}
